/*
 * Oscar Shell TUI
 * Copyright (c) 2004, Richard S. Hall
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the ungoverned.org nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Richard S. Hall (heavy@ungoverned.org)
 * Contributor(s):
 *
**/
package org.ungoverned.osgi.bundle.shelltui;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import org.osgi.framework.*;
import org.ungoverned.osgi.service.shell.ShellService;

public class ShellTuiActivator implements BundleActivator
{
    private transient BundleContext m_context = null;
    private transient ShellTuiRunnable m_runnable = null;
    private transient ServiceReference m_shellRef = null;
    private transient ShellService m_shell = null;

    public void start(BundleContext context)
    {
        m_context = context;

        // Listen for registering/unregistering shell service.
        ServiceListener sl = new ServiceListener() {
            public void serviceChanged(ServiceEvent event)
            {
                synchronized (ShellTuiActivator.this)
                {
                    // Ignore additional services if we already have one.
                    if ((event.getType() == ServiceEvent.REGISTERED)
                        && (m_shellRef != null))
                    {
                        return;
                    }
                    // Initialize the service if we don't have one.
                    else if ((event.getType() == ServiceEvent.REGISTERED)
                        && (m_shellRef == null))
                    {
                        initializeService();
                    }
                    // Unget the service if it is unregistering.
                    else if ((event.getType() == ServiceEvent.UNREGISTERING)
                        && event.getServiceReference().equals(m_shellRef))
                    {
                        m_context.ungetService(m_shellRef);
                        m_shellRef = null;
                        m_shell = null;
                        // Try to get another service.
                        initializeService();
                    }
                }
            }
        };
        try {
            m_context.addServiceListener(sl,
                "(objectClass="
                + org.ungoverned.osgi.service.shell.ShellService.class.getName()
                + ")");
        } catch (InvalidSyntaxException ex) {
            System.err.println("ShellTuiActivator: Cannot add service listener.");
            System.err.println("ShellTuiActivator: " + ex);
        }

        // Now try to manually initialize the shell service
        // since one might already be available.
        initializeService();

        // Start shell thread.
        new Thread(
            m_runnable = new ShellTuiRunnable(),
            "Oscar Shell TUI").start();
    }

    private synchronized void initializeService()
    {
        if (m_shell != null)
            return;
        m_shellRef = m_context.getServiceReference(
            org.ungoverned.osgi.service.shell.ShellService.class.getName());
        if (m_shellRef == null)
            return;
        m_shell = (ShellService) m_context.getService(m_shellRef);
    }

    public void stop(BundleContext context)
    {
        if (m_runnable != null)
        {
            m_runnable.stop();
        }
    }

    private class ShellTuiRunnable implements Runnable
    {
        private boolean stop = false;

        public void stop()
        {
            stop = true;
        }

        public void run()
        {
            String line = null;
            BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

            while (!stop)
            {
                System.out.print("-> ");

                try {
                    line = in.readLine();
                } catch (IOException ex) {
                    System.err.println("Could not read input, please try again.");
                    continue;
                }

                synchronized (ShellTuiActivator.this)
                {
                    if (m_shell == null)
                    {
                        System.out.println("No shell service available.");
                        continue;
                    }

                    if (line == null)
                    {
                        continue;
                    }

                    line = line.trim();

                    if (line.length() == 0)
                    {
                        continue;
                    }

                    try {
                        m_shell.executeCommand(line, System.out, System.err);
                    } catch (Exception ex) {
                        System.err.println("ShellTuiActivator: " + ex);
                        ex.printStackTrace();
                    }
                }
            }
        }
    }
}
